from Globals import InitializeClass
from AccessControl import ClassSecurityInfo
from Products.CMFDefault.File import File
from DateTime import DateTime

from Products.CMFCore import CMFCorePermissions

factory_type_information = (
    {'id': 'FlashMovie',
     'content_icon': 'flashmovie_icon.gif',
     'meta_type': 'CMF FlashMovie',
     'description': 'a simple FlashMovie product for CMF',
     'product': 'CMFFlashMovie',
     'factory': 'addFlashMovie',
     'immediate_view': 'flashmovie_edit_form',
     'actions': (                 { 'id'            : 'view'
                                  , 'name'          : 'View'
                                  , 'action'        : 'flashmovie_view'
                                  , 'permissions'   : (
                                      CMFCorePermissions.View, )
                                  }
                                , { 'id'            : 'edit'
                                  , 'name'          : 'Edit'
                                  , 'action'        : 'flashmovie_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                , { 'id'            : 'metadata'
                                  , 'name'          : 'Metadata'
                                  , 'action'        : 'metadata_edit_form'
                                  , 'permissions'   : (
                                      CMFCorePermissions.ModifyPortalContent, )
                                  }
                                )
     },
   )

import OFS.Image

def addFlashMovie(self
		, id
		, title=''
		, file=''
		, content_type='application/x-shockwave-flash'
        , precondition=()
		, subject=()
		, description=''
		, contributors=()
		, effective_date=None
		, expiration_date=None
		, format='application/x-shockwave-flash'
		, language=''
		, rights=''):
    """object manager constructor"""
    id, title = OFS.Image.cookId(id, title, file)
    self = self.this()

    o = FlashMovie(id, title, '', content_type, precondition, subject
               , description, contributors, effective_date, expiration_date
               , format, language, rights
               )
    self._setObject(id, o)
    if file:
        self._getOb(id).manage_upload(file)
  
# get unsigned bits
def getUBits(bitList, bitPos, nr):
    valD = 0
    for runD in xrange(bitPos, bitPos+nr):
	valD = valD*2 + bitList[runD]
    return valD

# get signed bits
def getSBits(bitList, bitPos, nr):
    valD = 0
    for runD in xrange(bitPos+1, bitPos+nr): #skip the first bit -> it is the sign
        valD = valD*2 + bitList[runD]
    if bitList[bitPos]: valD = -valD
    return valD


def analyseContent(data, filesize):
    
    if len(data)<=(8+17):
        raise "Content error","data too small!"
    # Uncompressed flash file
    if str(data[:3]) == 'FWS':
        compressed=0
    # Version 6 or greater ZLIB compressed flash file
    elif ((str(data[:3]) == 'CWS') or (str(data[:3]) == 'FWC')):
    	compressed=1    	
    else:
    	raise "Content error","This does not appear to be an SWF file!"
    	
    version = str(ord(data[3]))
    uncompressed_size = ord(data[4]) + ord(data[5])*256 + ord(data[6])*256*256 + ord(data[7])*256*256*256
    
    data=data[8:1024]
    if compressed:
        from zlib import decompressobj
        do = decompressobj()
        data=do.decompress(data)
        do.flush()
      
    bitList = []
    for runD in xrange(0, 17):
        byteD = ord(data[runD])
        bitList.append( (byteD&128)/128 )
        bitList.append( (byteD&64)/64 )
        bitList.append( (byteD&32)/32 )
        bitList.append( (byteD&16)/16 )
        bitList.append( (byteD&8)/8 )
        bitList.append( (byteD&4)/4 )
        bitList.append( (byteD&2)/2 )
        bitList.append( (byteD&1)/1 )
    bitPos = 0
    fieldSize = getUBits(bitList, bitPos, 5); bitPos=bitPos+ 5
    x1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    x2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y1 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    y2 = getSBits(bitList, bitPos, fieldSize); bitPos=bitPos+ fieldSize
    p=(bitPos/8+1)
    frameRateMinor=ord(data[p]); p+=1
    frameRateMajor=ord(data[p]); p+=1
    framesTotal=ord(data[p])+ord(data[p+1])*254    
    return {'flashversion':version 
           , 'compressed':compressed 
           , 'width': int(x2)/20 
           , 'height':int(y2)/20
           , 'uncompressed_size':uncompressed_size 
           , 'frame_rate':'%s.%s' % (frameRateMajor, frameRateMinor)
           , 'frames_total':framesTotal                       
           }

           
class FlashMovie(File):

    meta_type = 'CMF FlashMovie'
    
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    _properties = (
        {'id':'title', 	'type':'string', 'mode':'w'},
        {'id':'description', 'type':'text', 'mode':'w'},
        {'id':'width', 	'type':'int', 'mode':'w'},
        {'id':'height', 'type':'int', 'mode':'w'},
        
	    {'id':'play',          'type':'boolean', 'mode':'w'},
	    {'id':'menu', 	       'type':'boolean', 'mode':'w'},
	    {'id':'loop', 	       'type':'boolean', 'mode':'w'},
	    {'id':'devicefont',    'type':'boolean', 'mode':'w'},        
	    {'id':'swliveconnect', 'type':'boolean', 'mode':'w'},
        
	    {'id':'quality', 'type':'selection', 'select_variable' : 'getQuality','mode':'w'},
	    {'id':'wmode', 	 'type':'selection', 'select_variable' : 'getWmode','mode':'w'},
	    {'id':'align', 	 'type':'selection', 'select_variable' : 'getAlign','mode':'w'},
	    {'id':'scale', 	 'type':'selection', 'select_variable' : 'getScale','mode':'w'},
	    {'id':'salign',  'type':'selection', 'select_variable' : 'getSAlign','mode':'w'},
            
	    {'id':'bgcolor', 'type':'string', 'mode':'w'},
	    {'id':'base',    'type':'string', 'mode':'w'}, 
	    {'id':'allowscriptaccess',  'type':'selection', 'select_variable' : 'getAllowScriptAccess','mode':'w'},               
	)

   
    def __init__(self
                , id
                , title=''
                , file=''
                , content_type=''
                , precondition=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='application/x-shockwave-flash'
                , language='en-US'
                , rights=''
                ):
        File.__init__(self, id, title, file, content_type, precondition
                    , subject, description, contributors, effective_date
                    , expiration_date, format, language, rights=''
                    )
        self.width = 0
        self.height = 0
        self.swliveconnect = 0 #FlashMovie.getDefaultSwliveconnect
        self.play = 1 #FlashMovie.getDefaultPlay
        self.loop = 1 #FlashMovie.getDefaultLoop
        self.devicefont = 0 #FlashMovie.getDefaultDevicefont
        self.quality = FlashMovie.getDefaultQuality
        self.bgcolor = FlashMovie.getDefaultBgcolor
        self.scale = FlashMovie.getDefaultScale
        self.align = FlashMovie.getDefaultAlign
        self.salign = FlashMovie.getDefaultSAlign
        self.base = FlashMovie.getDefaultBase
        self.menu = 1 #FlashMovie.getDefaultMenu
        self.wmode = FlashMovie.getDefaultWMode
        self.allowscriptaccess = FlashMovie.getDefaultAllowScriptAccess                
        
        
    security.declareProtected(CMFCorePermissions.ModifyPortalContent, 'edit')
    def edit(self, precondition='', file='', title='', description='', width=0, height=0, 
                   swliveconnect=0, play=1, loop=1, devicefont=0, quality='high', bgcolor='',
                   scale='swowall', align='', salign='',base='', menu=1, wmode='windowed', allowscriptaccess='samedomain'):
        """ Update and reindex. """

        self.setTitle(title)
        self.setDescription(description)
        self.width = width
        self.height = height
        self.swliveconnect = swliveconnect
        self.play = play
        self.loop = loop
        self.devicefont = devicefont
        self.quality = quality
        self.bgcolor = bgcolor
        self.scale = scale
        self.align = align
        self.salign = salign
        self.base = base
        self.menu = menu
        self.wmode = wmode
        self.allowscriptaccess = allowscriptaccess
        self.reindexObject()
        self._edit(precondition, file)                

    def manage_upload(self,file='',REQUEST=None):
        """
        Upload an SWF file 
        """
        
        if self.wl_isLocked():
            raise ResourceLockedError, "File is locked via WebDAV"

        data, size = self._read_data(file)
        	
        self.update_data(data, self.content_type, size)

        file.seek(0)
        data=file.read()
        filesize=len(data)
        analysed=analyseContent(data,filesize)

        self.height=analysed['height']
        self.width=analysed['width']
        self.version=analysed['flashversion']
        self.compressed=analysed['compressed']
        self.uncompressed_size=analysed['uncompressed_size']
        self.frame_rate = analysed['frame_rate']
        self.frames_total = analysed['frames_total']
        self.content_type="application/x-shockwave-flash"
        if REQUEST:
            return self.manage_edit(self,REQUEST)

    #properties and values
    getQuality = ['low','autolow','autohigh','medium','high','best']
    getScale = ['showall','noborder','exactfit','noscale']
    getAlign = ['','l','r','t','b']
    getWmode = ['window','opaque','transparent']
    getAllowScriptAccess = ['always','never','samedomain']
    getSAlign = ['','l','lt','t','rt','r','rb','b','lb']
    
    getDefaultSwliveconnect = 'false'
    getDefaultPlay = 'true'
    getDefaultLoop = 'true'
    getDefaultDevicefont = 'false'
    getDefaultQuality = 'high'
    getDefaultBgcolor = ''
    getDefaultScale = 'showall'
    getDefaultAlign = ''
    getDefaultSAlign  = ''
    getDefaultBase = ''
    getDefaultMenu = 'true'
    getDefaultWMode = 'window'
    getDefaultAllowScriptAccess = 'samedomain'
    
    getQualityValues = [{'value':'low', 'label':'Low', }, 
             {'value':'autolow','label':'AutoLow',},
             {'value':'autohigh', 'label':'AutoHigh',},
             {'value':'medium', 'label':'Medium',}, 
             {'value':'high', 'label':'High',},
             {'value':'best', 'label':'Best',}
            ] 

    getScaleValues = [{'value':'showall', 'label':'Default (Show all)', }, 
             {'value':'noborder','label':'No border',},
             {'value':'exactfit', 'label':'Exact fit',},
             {'value':'noscale', 'label':'No scale',}, 
            ] 
      
    getHTMLAlignmentValues = [{'value':'', 'label':'Default', },
             {'value':'l', 'label':'Left', }, 
             {'value':'t','label':'Top',},
             {'value':'r', 'label':'Right',},
             {'value':'b', 'label':'Bottom',},             
            ]      
        
    getWindowModeValues = [{'value':'window', 'label':'Window', }, 
             {'value':'opaque','label':'Opaque Windowless',},
             {'value':'transparent', 'label':'Transparent Windowless',},
            ] 

    getAllowScriptAccessValues = [{'value':'always', 'label':'Always', }, 
             {'value':'never','label':'Never',},
             {'value':'samedomain', 'label':'Same Domain',},
            ] 

InitializeClass(FlashMovie)
        

